#include "processing/checkers/MinimalityChecker.hpp"

#include <vector>
#include <future>

#include "utils/concurrency.hpp"
#include "sfsm/TransitionRelationGenerator.hpp"

namespace libfsmtest::SFSMs {

bool MinimalityChecker::operator()(Sfsm const &sfsm) {
    std::vector<std::future<std::optional<ConcreteTrace>>> results;
    auto const &states = sfsm.getStates();
    for(size_t idx1 = 0; idx1 < states.size(); ++idx1) {
        for(size_t idx2 = idx1 + 1; idx2 < states.size(); ++idx2) {
            results.push_back(async(getDistinguishingTraceForSfsm(sfsm, idx1, idx2)));
        }
    }
    for(auto &result : results) {
        if(not result.get().has_value()) {
            return false;
        }
    }

    return true;
}

}

