#include "processing/checkers/ObservabilityChecker.hpp"

#include <vector>
#include <future>

#include "sfsm/Transition.hpp"
#include "sfsm/Expressions.hpp"
#include "sfsm/SMTTools.hpp"

#include "utils/concurrency.hpp"

namespace libfsmtest::SFSMs {

bool ObservabilityChecker::operator()(Sfsm const &sfsm) {
    std::vector<std::future<SatisfyingValuationResult>> results;
    auto guardConditions = sfsm.getGuardConditions();
    auto outputExpressions = sfsm.getOutputExpressions();
    for(auto const &state : sfsm.getStates()) {
        auto const &transitions = state.getTransitions();
        for(size_t idx1 = 0; idx1 != transitions.size(); ++idx1) {
            auto const &[preId1, guardId1, outId1, postId1] = transitions.at(idx1);
            auto const &guard1 = guardConditions.at(guardId1);
            auto const &out1 = outputExpressions.at(outId1);
            for(size_t idx2 = idx1 + 1; idx2 != transitions.size(); ++idx2) {
                auto const &[preId2, guardId2, outId2, postId2] = transitions.at(idx2);
                auto const &guard2 = guardConditions.at(guardId2);
                auto const &out2 = outputExpressions.at(outId2);
                auto expression = makeLogicalConjunction(guard1, guard2, out1, out2, sfsm.getInvariant());
                results.push_back(async(isSatisfiable(expression, sfsm.getIdentifierDefinitions())));
            }
        }
    }
    for(auto &result : results) {
        if(result.get().has_value()) {
            return false;
        }
    }

    return true;
}

}

