/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "processing/checkers/UnsatisfiableTransitionsChecker.hpp"
#include "sfsm/SMTTools.hpp"
#include "sfsm/Sfsm.hpp"
#include "sfsm/State.hpp"
#include "sfsm/Transition.hpp"
#include "sfsm/Expressions.hpp"

namespace libfsmtest::SFSMs {

UnsatisfiableTransitionsChecker::result_type
UnsatisfiableTransitionsChecker::operator()(Sfsm const &sfsm) {
    result_type foundTransitions;
    for(auto const &state : sfsm.getStates()) {
        for(auto const &transition : state.getTransitions()) {
            auto const guardCondition = sfsm.getGuardConditions().at(transition.getInputId());
            auto const outputExpression = sfsm.getOutputExpressions().at(transition.getOutputId());
            auto const expression = makeLogicalConjunction(
                                        guardCondition,
                                        outputExpression,
                                        sfsm.getInvariant()
                                    );
            if(not isSatisfiable(expression, sfsm.getIdentifierDefinitions())) {
                foundTransitions.push_back(&transition);
            }
        }
    }
    return foundTransitions;
}

} // namespace libfsmtest::SFSMs

