/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "processing/transformations/ToFSMAbstraction.hpp"

#include "sfsm/State.hpp"
#include "sfsm/Transition.hpp"

std::unique_ptr<libfsmtest::Fsm> libfsmtest::SFSMs::ToFSMAbstraction::operator()(const libfsmtest::SFSMs::Sfsm &input) {
    libfsmtest::States fsmStates;
    for(auto const &state : input.getStates()) {
        fsmStates.emplace_back(state.getName(), state.getId());
        for(auto const &transition : state.getTransitions()) {
            fsmStates.back().addTransition(transition.getInputId(), transition.getOutputId(), transition.getTargetId());
        }
    }

    std::vector<std::string> inputStrings;
    std::transform(input.getUsedGuardConditions().cbegin(), input.getUsedGuardConditions().cend(), std::back_inserter(inputStrings), static_cast<std::string(*)(libfsmtest::SFSMs::Expression const&)>(std::to_string));
    std::vector<std::string> outputStrings;
    std::transform(input.getUsedOutputExpressions().cbegin(), input.getUsedOutputExpressions().cend(), std::back_inserter(outputStrings), static_cast<std::string(*)(libfsmtest::SFSMs::Expression const&)>(std::to_string));
    std::vector<std::string> stateStrings;
    std::transform(input.getStates().cbegin(), input.getStates().cend(), std::back_inserter(stateStrings), [](auto const &state) { return std::to_string(state.getId()); });
    libfsmtest::PresentationLayer presentationLayer(inputStrings, outputStrings, stateStrings);
    return std::make_unique<libfsmtest::Fsm>("fsmAbstraction", fsmStates, presentationLayer, input.getInitialStateId());
}
