/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#pragma once

#include <optional>
#include <unordered_map>
#include <future>

#include "sfsm/ExpressionDefinitions.hpp"
#include "sfsm/Expressions.hpp"

namespace libfsmtest::SFSMs {

typedef std::optional<IdentifierValuations> SatisfyingValuationResult;
typedef std::optional<std::unordered_map<Expression, Expression>> QueryResult;

[[nodiscard]] QueryResult isSatisfiable(Expressions const &expressions, Expressions const &queries = {});
//[[nodiscard]] std::future<QueryResult> asyncIsSatisfiable(Expressions const &expressions, Expressions const &queries = {});
[[nodiscard]] SatisfyingValuationResult isSatisfiable(Expression const &expression, IdentifierDefinitions const &identifierDefinitions);
//[[nodiscard]] std::future<SatisfyingValuationResult> asyncIsSatisfiable(Expression const &expression, IdentifierDefinitions const &identifierDefinitions);

[[nodiscard]] Expression makeIdentifierDefinition(IdentifierDefinition const &id);
[[nodiscard]] Expression makeFunctionDefinition(Expression const &functionIdentifier, IdentifierType const &returnType, IdentifierDefinitions const &parameters, Expression const &content);
[[nodiscard]] Expression makeRecursiveFunctionDefinition(Expression const &functionIdentifier, IdentifierType const &returnType, IdentifierDefinitions const &parameters, Expression const &content);
[[nodiscard]] Expression makeFunctionDeclaration(IdentifierDefinition const &definition);
[[nodiscard]] Expression makeUniversallyQuantifiedFormula(IdentifierDefinitions const &quantifiedIdentifiers, Expression const &content);

// Convenience functions for common checks
[[nodiscard]] bool areEquivalent(Expression const &exp1, Expression const &exp2, IdentifierDefinitions const &identifierDefinitions);
[[nodiscard]] bool isImplication(Expression const &antecedent, Expression const &consequent, IdentifierDefinitions const &identifierDefinitions);

Expression getSMTTypeExpression(IdentifierType const &type);

}

