/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#pragma once

#include <unordered_map>
#include <utility>
#include <optional>
#include <vector>

#include "sfsm/ExpressionDefinitions.hpp"
#include "sfsm/Expressions.hpp"
#include "sfsm/definitions.hpp"
#include "sfsm/Variable.hpp"
#include "sfsm/Sfsm.hpp"

namespace libfsmtest::SFSMs {
class Transition;
class State;

Expression generateTransitionRelationForTransition(IdentifierDefinitions const &definitions, Transition const &transition);
Expression generateTransitionRelationForState(IdentifierDefinitions const &definitions, State const &state);
Expression generateTransitionRelationForSfsm(Sfsm const &sfsm);
Expression generateTransitionRelationDefinitionForSfsm(Sfsm const &sfsm);

// Expects arguments for the transition relation, where each argument
// corresponds to one identifier in the SFSM. They are expected in the order
// specified by sfsm.getIdentifierDefinitions()
Expression getTransitionRelationCall(Sfsm const &sfsm, Expressions const &arguments);

std::pair<std::unordered_map<IdentifierDefinition, IdentifierDefinition>, Expressions> findDistinguishingTraceProblem(Sfsm const &sfsm, Expression const &transitionRelationDefinition, Index state1, Index state2);
std::optional<ConcreteTrace> getDistinguishingTrace(Sfsm const &sfsm, Expression const &transitionRelationDefinition, Index state1, Index state2);
std::pair<std::unordered_map<IdentifierDefinition, IdentifierDefinition>, Expressions> findDistinguishingTraceProblem(Sfsm const &sfsm, Expression const &transitionRelationDefinition, std::vector<Index> const &state1, std::vector<Index> const &state2);
std::optional<ConcreteTrace> getDistinguishingTrace(Sfsm const &sfsm, Expression const &transitionRelationDefinition, std::vector<Index> state1, std::vector<Index> state2);

std::optional<ConcreteTrace> getDistinguishingTraceForSfsm(Sfsm const &sfsm, Index state1, Index state2);
std::optional<ConcreteTrace> getDistinguishingTraceForAbstractedSfsm(Sfsm const &sfsm, Expressions const &propositionEquivalenceClasses, Index state1, Index state2);
std::optional<ConcreteTrace> getDistinguishingTraceForSfsm(Sfsm const &sfsm, ConcreteTrace const &trace1, ConcreteTrace const &trace2);
std::optional<ConcreteTrace> getDistinguishingTraceForAbstractedSfsm(Sfsm const &sfsm, Expressions const &propositionEquivalenceClasses, ConcreteTrace const &trace1, ConcreteTrace const &trace2);

std::optional<ConcreteTrace> getDistinguishingTraceForSfsm(Sfsm const &sfsm, std::vector<Index> const &state1, std::vector<Index> const &state2);
std::optional<ConcreteTrace> getDistinguishingTraceForAbstractedSfsm(Sfsm const &sfsm, Expressions const &propositionEquivalenceClasses, std::vector<Index> const &state1, std::vector<Index> const &state2);

std::vector<Index> getTargetStatesForSfsm(Sfsm const &sfsm, std::vector<Index> const &initialStates, ConcreteTrace const &trace);
std::vector<Index> getTargetStatesForSfsm(Sfsm const &sfsm, std::vector<Index> const &initialStates, SymbolicTrace const &trace);
std::vector<Index> getTargetStatesForAbstractedSfsm(Sfsm const &sfsm, Expressions const &propositionEquivalenceClasses, std::vector<Index> const &initialStates, ConcreteTrace const &trace);

std::pair<std::vector<Index>, std::vector<Index>> generateStatePairingsForDistinguishability(std::vector<Index> states1, std::vector<Index> states2);


}
