/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#pragma once

#include <vector>
#include <string>

#include "sfsm/ExpressionDefinitions.hpp"
#include "sfsm/Expressions.hpp"

namespace libfsmtest::SFSMs {

/**
 * Class representing SFSM valuations
 */
class Valuation {
public:
    typedef IdentifierValuation value_type;
private:
    ValuationFunction valuationFunction;
public:
    Valuation() = default;
    Valuation(Valuation const &) = default;
    Valuation(Valuation &&) = default;
    explicit Valuation(ValuationFunction paramValuationFunction) : valuationFunction(std::move(paramValuationFunction)) { }
    explicit Valuation(IdentifierValuations const &valuations);
    // Valuation(std::initializer_list<IdentifierValuation> valuationInit) : valuationFunction(valuationInit) { }
    ~Valuation() = default;

    Valuation &operator=(Valuation const&) = default;
    bool operator<(Valuation const &other) const;
    bool operator==(Valuation const &other) const;

    [[nodiscard]] ValuationFunction const& getValuations() const { return this->valuationFunction; }
    [[nodiscard]] ValuationFunction getValuations(IdentifierDefinitions const &variableSet) const;
    [[nodiscard]] Value const& getValuation(IdentifierDefinition const &variable) const;

    [[nodiscard]] Valuation modify(IdentifierDefinition const &variable, Value const &value) const;

    [[nodiscard]] decltype(auto) begin() const { return this->valuationFunction.begin(); }
    [[nodiscard]] decltype(auto) cbegin() const { return this->valuationFunction.begin(); }
    [[nodiscard]] decltype(auto) end() const { return this->valuationFunction.end(); }
    [[nodiscard]] decltype(auto) cend() const { return this->valuationFunction.end(); }

    [[nodiscard]] Expression toExpression() const;
};

typedef std::vector<Valuation> ConcreteTrace;

Valuation getValuationFromExpression(Expression const &content, IdentifierDefinitions const &identifiers);
std::variant<bool,ssize_t,double> getValueInType(IdentifierValuation const &valuation);

} // namespace libfsmtest

namespace std {
[[nodiscard]] std::string to_string(libfsmtest::SFSMs::Valuation const &valuation);
[[nodiscard]] std::string to_string(libfsmtest::SFSMs::ConcreteTrace const &concreteTrace);
template<>
struct hash<libfsmtest::SFSMs::ValuationFunction> {
    size_t operator()(libfsmtest::SFSMs::ValuationFunction const &obj) const;
};
template<>
struct hash<libfsmtest::SFSMs::Valuation> {
    size_t operator()(libfsmtest::SFSMs::Valuation const &obj) const;
};
}

