/* MIT License
 * 
 * Copyright (c) 2022 Niklas Krafczyk
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/* type lexer */

%{

#include <string>
#include <iostream>
#include <stdexcept>
#include <system_error>

#include "sfsm/typeparser/TypeParser.hpp"

#include "sfsm/Expressions.hpp"
#include "sfsm/typeparser/TypeParser.tab.hpp"
#define YYSTYPE SFSMIDENTIFIERTYPESTYPE

%}

/* Options */
%option nomain
%option noyywrap
%option nounput
%option noinput
%option case-insensitive
%option prefix="sfsmIdentifierType"
%option reentrant
%option bison-bridge
%option warn
%option nodefault

/* Symbols */

ARRAY "array"
REAL "real"
WHITESPACE [ \t\n]+

%%

"in" |
"input" { return IN; }

"out" |
"output" { return OUT; }

ARRAY { return ARRAY; }

"int" |
"integer" { return INTEGER; }

REAL { return REAL; }

"bool" |
"boolean" { return BOOL; }

"," { return COMMA; }
"(" { return LPAREN; }
")" { return RPAREN; }

{WHITESPACE} { }
. { }

%%

void sfsmIdentifierTypeerror(libfsmtest::SFSMs::IdentifierType *&/*result*/, std::string &errorString, yyscan_t /*scanner*/, char const *str) {
		errorString = str;
}

namespace libfsmtest::SFSMs {

libfsmtest::SFSMs::IdentifierType parseIdentifierType(std::string const &input) {
    libfsmtest::SFSMs::IdentifierType *resultPtr;
    std::string errorString;
    yyscan_t scanner;
    if(yylex_init(&scanner)) {
        throw std::system_error(errno, std::system_category());
    }
    auto buffer = yy_scan_string(input.c_str(), scanner);
    auto parseResult = sfsmIdentifierTypeparse(resultPtr, errorString, scanner);
    yy_delete_buffer(buffer, scanner);
    yylex_destroy(scanner);
    if(parseResult != 0) {
        throw std::runtime_error(std::string("Could not parse input as identifier type: ") + errorString);
    }
    libfsmtest::SFSMs::IdentifierType result = *resultPtr;
    delete resultPtr;
    return result;
}

}

