
/*
 * Copyright 2022 Niklas Krafczyk, Jan Peleska
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#pragma once

#include <optional>
#include <vector>

namespace libfsmtest::SFSMs {

    // Make sure the searched element is reachable. Otherwise, this will not return.
    template<typename ElementType, typename ElementGetterFunction>
    std::optional<std::vector<ElementType>> breadthFirstSearch(ElementGetterFunction const &getterFunction, ElementType const &initialElement, ElementType const &elementToSearchFor) {
        if(initialElement == elementToSearchFor) {
            return std::vector<ElementType>{};
        }
        std::vector<std::vector<ElementType>> current, next{{initialElement}};
        while(current != next) {
            current = next;
            for(auto const &singlePath : current) {
                auto const nextElements = getterFunction(singlePath.back());
                for(auto const &nextElement : nextElements) {
                    auto singlePathExtended = singlePath;
                    singlePathExtended.emplace_back(nextElement);
                    if(nextElement == elementToSearchFor) {
                        return singlePathExtended;
                    } else {
                        next.push_back(singlePathExtended);
                    }
                }
            }
        }
        return std::nullopt;
    }
}

